#include <iostream>
#include <fstream>
#include <csignal>
#include "common.h"
#include "ArgParser.h"
#include "Logger.h"
#include "Parser.h"
#include "Interpreter.h"

int main(int argc, char* argv[]) {
    // Parse and act upon command line arguments
    ArgParser args(argc, argv);
    // Initialise the logger
    Logger log;
    if (debugMode) log.toggleDebugPrint();
    log.debug("Logger initialised!");
    // Initialise the parser
    Parser parser;
    // Exit if file doesn't exist
    if (argc < 2) log.fatalError("Please provide a file", 1);
    // Read the file
    ifstream inputFile(args.getArg(1));
    string input;
    string file;
    while(getline(inputFile, input)) {
        file += input;
    }
    inputFile.close();
    // Parse the file
    parser.parseLines(file, log);
    parser.processLines();
    // Initialise the interpreter
    Interpreter interpreter;
    // Convert to tokens and run the code
    interpreter.convertToTokens(parser.getTokens());
    return 0;
}